'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var one = require('remax/one');
var ali = require('remax/ali');
var wechat = require('remax/wechat');
var toutiao = require('remax/toutiao');

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var ScrollViewWrapper;

if (process.env.REMAX_PLATFORM === 'ali') {
  ScrollViewWrapper = ali.ScrollView;
} else if (process.env.REMAX_PLATFORM === 'wechat') {
  ScrollViewWrapper = wechat.ScrollView;
} else if (process.env.REMAX_PLATFORM === 'toutiao') {
  ScrollViewWrapper = toutiao.ScrollView;
} else {
  throw new Error("current platform ".concat(process.env.REMAX_PLATFORM, " is unknown"));
}

var ScrollViewRender = function ScrollViewRender(props, ref) {
  var children = props.children,
      scrollViewProps = _objectWithoutProperties(props, ["children"]);

  return React.createElement(ScrollViewWrapper, Object.assign({}, scrollViewProps, {
    ref: ref
  }), children);
};

var ScrollView = React.forwardRef(ScrollViewRender);

function getSystemInfo() {
  if (process.env.REMAX_PLATFORM === 'ali') {
    return ali.getSystemInfoSync();
  }

  if (process.env.REMAX_PLATFORM === 'wechat') {
    return wechat.getSystemInfoSync();
  }
  /* istanbul ignore else */


  if (process.env.REMAX_PLATFORM === 'toutiao') {
    return toutiao.getSystemInfoSync();
  } // this case has been thrown before

  /* istanbul ignore next */


  throw new Error("current platform ".concat(process.env.REMAX_PLATFORM, " is unknown"));
}

var systemInfo = getSystemInfo();

/**
 * set visible range for recycle-view
 * @param overscanCount the overscan count above or below offset
 */

function useVisibleRange(overscanCount) {
  var _React$useState = React.useState(0 - overscanCount),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      start = _React$useState2[0],
      setStart = _React$useState2[1];

  var _React$useState3 = React.useState(0 + 2 * overscanCount),
      _React$useState4 = _slicedToArray(_React$useState3, 2),
      end = _React$useState4[0],
      setEnd = _React$useState4[1];

  var setRange = React.useCallback(function (offset) {
    setStart(offset - overscanCount);
    setEnd(offset + 2 * overscanCount);
  }, [setStart, setEnd]);
  return [start, end, setRange];
}
/**
 * get size map array include height and offsetTop, in addition it will add __index__ to data
 * @param data raw data to get size map array
 */

function useSizeData(data) {
  return React.useMemo(function () {
    var offsetTop = 0;
    return data.map(function (_ref, index) {
      var height = _ref.height;
      offsetTop += height;
      data[index].__index__ = index;
      return {
        height: height,
        offsetTop: offsetTop - height
      };
    });
  }, [data]);
}

function throttle(func, wait) {
  var previous = 0;
  var time;
  var remaining;
  return function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var now = +new Date();
    var context = this;
    remaining = wait - (now - previous);

    if (remaining <= 0) {
      func.apply(context, args);
      previous = now;
    } else {
      if (time) {
        clearTimeout(time);
      }

      time = setTimeout(function () {
        func.apply(context, args);
        time = undefined;
        previous = +new Date();
      }, remaining);
    }
  };
}

var DEFAULT_OVERSCAN_COUNT = 5;

var isVisible = function isVisible(start, end) {
  return function (_, index) {
    return index >= start && index <= end;
  };
};

var RecycleView = function RecycleView(props, ref) {
  var _props$data = props.data,
      data = _props$data === void 0 ? [] : _props$data,
      _props$overscanCount = props.overscanCount,
      overscanCount = _props$overscanCount === void 0 ? DEFAULT_OVERSCAN_COUNT : _props$overscanCount,
      placeholderImage = props.placeholderImage,
      renderHeader = props.renderHeader,
      renderBottom = props.renderBottom,
      _props$headerHeight = props.headerHeight,
      headerHeight = _props$headerHeight === void 0 ? 0 : _props$headerHeight,
      _props$bottomHeight = props.bottomHeight,
      bottomHeight = _props$bottomHeight === void 0 ? 0 : _props$bottomHeight,
      renderItem = props.renderItem,
      _onScroll = props.onScroll,
      scrollViewProps = _objectWithoutProperties(props, ["data", "overscanCount", "placeholderImage", "renderHeader", "renderBottom", "headerHeight", "bottomHeight", "renderItem", "onScroll"]);

  var _useVisibleRange = useVisibleRange(overscanCount),
      _useVisibleRange2 = _slicedToArray(_useVisibleRange, 3),
      start = _useVisibleRange2[0],
      end = _useVisibleRange2[1],
      setRange = _useVisibleRange2[2];

  var LIST_HEIGHT = React.useMemo(function () {
    return data.reduce(function (totalHeight, ele) {
      return totalHeight + ele.height;
    }, 0);
  }, [data]);
  var sizeData = useSizeData(data);
  var visibleData = React.useMemo(function () {
    return data.filter(isVisible(start, end));
  }, [data, start, end]);
  var handleScroll = throttle(function (event) {
    var _event$detail = event.detail,
        scrollTop = _event$detail.scrollTop,
        scrollHeight = _event$detail.scrollHeight;
    var ratio = scrollTop / scrollHeight;
    var initialOffsetTop = headerHeight;
    var totalHeight = LIST_HEIGHT + bottomHeight;
    var offset = 0;

    for (var i = 0; i < sizeData.length; i++) {
      var offsetTop = sizeData[i].offsetTop;
      var totalOffsetTop = initialOffsetTop + offsetTop;

      if (totalOffsetTop / totalHeight >= ratio) {
        offset = i;
        break;
      }
    }

    setRange(offset);
  }, 100);
  var innerBeforeHeight = sizeData[start] && sizeData[start].offsetTop || 0;
  return React.createElement(ScrollView, Object.assign({}, scrollViewProps, {
    scrollY: true,
    ref: ref,
    onScroll: function onScroll(e) {
      handleScroll(e);
      _onScroll && _onScroll(e);
    }
  }), renderHeader && React.createElement(one.View, {
    className: "recycle-view-header",
    style: {
      height: headerHeight
    }
  }, renderHeader()), React.createElement(one.View, {
    style: {
      position: 'relative',
      height: LIST_HEIGHT,
      background: placeholderImage && "url(\"".concat(placeholderImage, "\") repeat-y"),
      backgroundSize: placeholderImage && 'contain'
    }
  }, React.createElement(one.View, {
    style: {
      position: 'absolute',
      left: 0,
      width: '100%',
      top: innerBeforeHeight
    }
  }, visibleData.map(function (item, index) {
    return React.createElement(one.View, {
      key: item.__index__,
      style: {
        height: item.height
      },
      className: "recycle-view-item"
    }, renderItem(item, index));
  }))), renderBottom && React.createElement(one.View, {
    className: "recycle-view-bottom",
    style: {
      height: bottomHeight
    }
  }, renderBottom()));
};

var index = React.forwardRef(RecycleView);

module.exports = index;
